/*
 * Copyright (C) 2010  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#pragma once

#include "configuration.h"

namespace plexdata
{
	namespace autoplay
	{
		class CButton
		{
		private:

			static const COLORREF NORMAL_BORDER;
			static const COLORREF HIGHLIGHT_BORDER;
			static const COLORREF FOCUSED_BORDER;
			static const int      BORDER_MARGIN;

		private:

			HWND          m_hButton;
			HWND          m_hParent;
			HWND          m_hTooltip;
			HBITMAP       m_hImage;
			HFONT         m_hFont;
			HCURSOR       m_hCursor;
			WNDPROC       m_pPrevHandler;
			volatile bool m_bGotFocus;
			volatile bool m_bTrackLeave;
			int           m_nAlignment;
			int           m_nButtonID;
			plexdata::string::CString m_sLabel;

		public:
			
			CButton();
			virtual ~CButton();

		public:

			bool Create( HWND hParent, int nButtonID, CButtonData* pButton, int x, int y, int cx, int cy );

			bool SetFont( LPCTSTR szFont, int nSize, bool bBold, int nAlignment );

		private:

			static LRESULT CALLBACK EventHandler( HWND hButton, UINT uMsg, WPARAM wParam, LPARAM lParam );

			void OnPaint( HDC hdcTarget );
			
			void OnMouseMove( int xPos, int yPos );

			void OnMouseLeave();

			void OnSetFocus();

			void OnKillFocus();

			void OnKeyDown( unsigned int nVirtualKey );

		private:

			void DrawBorder();

			void DrawImage( HDC hdcTarget, int x, int y, int cx, int cy );
		};
	}
}