/*
 * Copyright (C) 2010  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "stdafx.h"
#include "configuration.h"
#include "application.h"

using namespace plexdata::autoplay;

//////////////////////////////////////////////////////////////////////
// 
// CButtonData
//
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
CButtonData::CButtonData()
	: m_nAlignment(DT_CENTER)
{
};
			
//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
CButtonData::~CButtonData()
{
};

//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
HBITMAP CButtonData::LoadBitmap()
{
	HBITMAP hImage = NULL;

	plexdata::string::CString sFile = CApplication::GetInstance().ModulePath();
	sFile += this->m_sImage;

	hImage = (HBITMAP)::LoadImage
		( NULL
		, sFile.String()
		, IMAGE_BITMAP
		, 0
		, 0
		, LR_LOADFROMFILE
		);

	return hImage;
}

//////////////////////////////////////////////////////////////////////
// 
// CConfiguration
//
//////////////////////////////////////////////////////////////////////

const LPCTSTR CConfiguration::EMPTY        = _T("");
const LPCTSTR CConfiguration::APPLICATION  = _T("application");
const LPCTSTR CConfiguration::TITLE        = _T("title");
const LPCTSTR CConfiguration::BACKGROUND   = _T("background");
const LPCTSTR CConfiguration::WELCOME      = _T("welcome");
const LPCTSTR CConfiguration::ICON         = _T("icon");
const LPCTSTR CConfiguration::ALIGNMENT    = _T("alignment");
const LPCTSTR CConfiguration::FONTNAME     = _T("fontname");
const LPCTSTR CConfiguration::FONTSIZE     = _T("fontsize");
const LPCTSTR CConfiguration::FONTCOLOR    = _T("fontcolor");
const LPCTSTR CConfiguration::FONTBOLD     = _T("fontbold");
const LPCTSTR CConfiguration::BUTTONSIZE   = _T("buttonsize");
const LPCTSTR CConfiguration::BORDER       = _T("border");
const LPCTSTR CConfiguration::BUTTON       = _T("button%d");
const LPCTSTR CConfiguration::LABEL        = _T("label");
const LPCTSTR CConfiguration::TOOLTIP      = _T("tooltip");
const LPCTSTR CConfiguration::IMAGE        = _T("image");
const LPCTSTR CConfiguration::EXECUTE      = _T("execute");
const LPCTSTR CConfiguration::PARAMETER    = _T("parameter");
const LPCTSTR CConfiguration::AFTERWARDS   = _T("afterwards");

const int CConfiguration::ALIGNMENT_TOP    = 0x00000001;
const int CConfiguration::ALIGNMENT_BOTTOM = 0x00000002;
const int CConfiguration::ALIGNMENT_LEFT   = 0x00000010;
const int CConfiguration::ALIGNMENT_RIGTH  = 0x00000020;
const int CConfiguration::ALIGNMENT_CENTER = 0x00000100;

const int CConfiguration::BORDER_STYLE_DEFAULT = 0x00000000;
const int CConfiguration::BORDER_STYLE_TOOL    = 0x00000001;
const int CConfiguration::BORDER_STYLE_THICK   = 0x00000002;
const int CConfiguration::BORDER_STYLE_THIN    = 0x00000003;
const int CConfiguration::BORDER_STYLE_NONE    = 0x00000004;

//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
CConfiguration::CConfiguration()
	: m_hBackground( NULL )
	, m_hIcon      ( NULL )
	, m_nVertical  ( CConfiguration::ALIGNMENT_CENTER )
	, m_nHorizontal( CConfiguration::ALIGNMENT_CENTER )
	, m_sFontName  ( _T("Times New Roman") )
	, m_nFontSize  ( 11 )
	, m_nFontColor ( COLORREF( RGB( 0, 0, 0 ) ) )
	, m_bFontBold  ( false )
	, m_pButtons   ( NULL )
	, m_nButtons   ( 0 )
	, m_nBorder    ( CConfiguration::BORDER_STYLE_DEFAULT )
{
	this->m_btnSize.cx = 200;
	this->m_btnSize.cy = 42;
}

//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
CConfiguration::~CConfiguration()
{
	::DeleteObject( this->m_hBackground );
	::DestroyIcon( this->m_hIcon );
	delete[] this->m_pButtons;
}

//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
void CConfiguration::Load( LPCTSTR szIniFile, int nIconID )
{
	// Reset current content.
	this->SetupDefaults();

	//
	// Read TITLE value!
	//
	this->m_sTitle = this->ReadValue
		( CConfiguration::APPLICATION
		, CConfiguration::TITLE
		, this->m_sTitle.String()
		, szIniFile
		);

	//
	// Read WELCOME value!
	//
	this->m_sWelcome = this->ReadValue
		( CConfiguration::APPLICATION
		, CConfiguration::WELCOME
		, this->m_sWelcome.String()
		, szIniFile
		);

	//
	// Read and prepare BACKGROUND value!
	//
	plexdata::string::CString sBackground = this->ReadValue
		( CConfiguration::APPLICATION
		, CConfiguration::BACKGROUND
		, CConfiguration::EMPTY
		, szIniFile
		);
	
	if ( !sBackground.IsEmpty() )
	{
		plexdata::string::CString sFile;

		int nSeparator = sBackground.FindFirst( _T(',') );
		plexdata::string::CString sHelper = sBackground.Substring( 0, nSeparator ).Trim();

		if ( sHelper.Equals(_T("RANDOM")))
		{
			plexdata::string::CString sPath = CApplication::GetInstance().ModulePath();
			sPath += sBackground.Substring( nSeparator + 1, -1 ).Trim();
			sFile = this->GetRandomFile( sPath );
		}
		else
		{
			sFile = CApplication::GetInstance().ModulePath();
			sFile += sBackground;
		}

		if ( !sFile.IsEmpty() )
		{
			this->m_hBackground = (HBITMAP)::LoadImage
				( NULL
				, sFile.String()
				, IMAGE_BITMAP
				, 0
				, 0
				, LR_LOADFROMFILE
				);
		}
	}

	//
	// Read and prepare ICON value!
	//
	plexdata::string::CString sIcon = this->ReadValue
		( CConfiguration::APPLICATION
		, CConfiguration::ICON
		, CConfiguration::EMPTY
		, szIniFile
		);

	// Load default icon if custom icon is not used.
	if ( sIcon.IsEmpty() )
	{
		this->m_hIcon = ::LoadIcon( CApplication::GetInstance().InstanceHandle(), MAKEINTRESOURCE( nIconID ) );
	}
	// Try to load custom icon.
	else if ( !sIcon.Equals( _T("OFF") ) )
	{
		plexdata::string::CString sFile = CApplication::GetInstance().ModulePath();
		
		sFile += sIcon;

		this->m_hIcon = (HICON)::LoadImage
			( NULL
			, sFile.String()
			, IMAGE_ICON
			, 0
			, 0
			, LR_LOADFROMFILE
			);

		// Use default icon if icon load from file faild!
		if ( this->m_hIcon == NULL )
		{
			this->m_hIcon = ::LoadIcon( CApplication::GetInstance().InstanceHandle(), MAKEINTRESOURCE( nIconID ) );
		}
	}
	
	//
	// Read and prepare ALIGNMENT values!
	//
	plexdata::string::CString sAlignment = this->ReadValue
		( CConfiguration::APPLICATION
		, CConfiguration::ALIGNMENT
		, CConfiguration::EMPTY
		, szIniFile
		);
	
	if ( !sAlignment.IsEmpty() )
	{
		int nSeparator = sAlignment.FindFirst( _T(',') );

		plexdata::string::CString sVertical = sAlignment.Substring( 0, nSeparator ).Trim();

		if ( sVertical.Equals( _T("TOP") ) )
		{
			this->m_nVertical = CConfiguration::ALIGNMENT_TOP;
		}
		else if ( sVertical.Equals( _T("CENTER") ) )
		{
			this->m_nVertical = CConfiguration::ALIGNMENT_CENTER;
		}
		else if ( sVertical.Equals( _T("BOTTOM") ) )
		{
			this->m_nVertical = CConfiguration::ALIGNMENT_BOTTOM;
		}
		else
		{
			// Default already set in function SetupDefaults().
		}

		plexdata::string::CString sHorizontal = sAlignment.Substring( nSeparator + 1, -1 ).Trim();

		if ( sHorizontal.Equals( _T("LEFT") ) )
		{
			this->m_nHorizontal = CConfiguration::ALIGNMENT_LEFT;
		}
		else if ( sHorizontal.Equals( _T("CENTER") ) )
		{
			this->m_nHorizontal = CConfiguration::ALIGNMENT_CENTER;
		}
		else if ( sHorizontal.Equals( _T("RIGHT") ) )
		{
			this->m_nHorizontal = CConfiguration::ALIGNMENT_RIGTH;
		}
		else
		{
			// Default already set in function SetupDefaults().
		}
	}

	//
	// Read FONTNAME value!
	//
	plexdata::string::CString sFontName = this->ReadValue
		( CConfiguration::APPLICATION
		, CConfiguration::FONTNAME
		, CConfiguration::EMPTY
		, szIniFile
		);

	if ( !sFontName.IsEmpty() )
	{
		this->m_sFontName = sFontName;
	}

	//
	// Read FONTSIZE value!
	//
	int nFontSize = _tstoi( this->ReadValue
		( CConfiguration::APPLICATION
		, CConfiguration::FONTSIZE
		, CConfiguration::EMPTY
		, szIniFile
		).String()
		);

	if ( 6 <= nFontSize )
	{
		this->m_nFontSize = nFontSize;
	}

	//
	// Read FONTCOLOR value!
	//
	plexdata::string::CString sFontColor = this->ReadValue
		( CConfiguration::APPLICATION
		, CConfiguration::FONTCOLOR
		, CConfiguration::EMPTY
		, szIniFile
		);

	if ( !sFontColor.IsEmpty() )
	{
		sFontColor.Trim();

		if ( sFontColor.Equals( _T("BLACK") ) )
		{
			this->m_nFontColor = COLORREF(RGB(0,0,0));
		}
		else if ( sFontColor.Equals( _T("WHITE") ) )
		{
			this->m_nFontColor = COLORREF(RGB(255,255,255));
		}
		else if ( sFontColor.Equals( _T("GRAY") ) )
		{
			this->m_nFontColor = COLORREF(RGB(128,128,128));
		}
		else if ( sFontColor.Equals( _T("RED") ) )
		{
			this->m_nFontColor = COLORREF(RGB(255,0,0));
		}
		else if ( sFontColor.Equals( _T("GREEN") ) )
		{
			this->m_nFontColor = COLORREF(RGB(0,255,0));
		}
		else if ( sFontColor.Equals( _T("BLUE") ) )
		{
			this->m_nFontColor = COLORREF(RGB(0,0,255));
		}
		else if ( sFontColor.Equals( _T("YELLOW") ) )
		{
			this->m_nFontColor = COLORREF(RGB(255,255,0));
		}
		else // RGB(r,g,b)
		{
			sFontColor = sFontColor.Substring
				( sFontColor.FindFirst( _T('(') ) + 1
				, sFontColor.FindLast( _T(')') ) 
				);

			long comma1 = sFontColor.FindFirst( _T(',') );
			long comma2 = sFontColor.FindNext( _T(','), comma1 + 1 );

			this->m_nFontColor = COLORREF
				( RGB
					( _tstoi( sFontColor.Substring( 0,          comma1 ).Trim().String() )
					, _tstoi( sFontColor.Substring( comma1 + 1, comma2 ).Trim().String() )
					, _tstoi( sFontColor.Substring( comma2 + 1, -1     ).Trim().String() )
					)
				);

			int i=0;
		}
	}

	//
	// Read FONTBOLD value!
	//
	plexdata::string::CString sFontBold = this->ReadValue
		( CConfiguration::APPLICATION
		, CConfiguration::FONTBOLD
		, CConfiguration::EMPTY
		, szIniFile
		);

	this->m_bFontBold = sFontBold.Equals( _T("true") ) || sFontBold.Equals( _T("yes") ) || sFontBold.Equals( _T("1") );

	//
	// Read BUTTONSIZE value!
	//
	plexdata::string::CString sButtonSize = this->ReadValue
		( CConfiguration::APPLICATION
		, CConfiguration::BUTTONSIZE
		, CConfiguration::EMPTY
		, szIniFile
		);
	
	if ( !sButtonSize.IsEmpty() )
	{
		int nSeparator = sButtonSize.FindFirst( _T(',') );

		int nWidth = _tstoi( sButtonSize.Substring( 0, nSeparator ).Trim().String() );
		if ( 0 < nWidth )
		{
			this->m_btnSize.cx = nWidth;
		}

		int nHeight = _tstoi( sButtonSize.Substring( nSeparator + 1, -1 ).Trim().String() );
		if ( 0 < nHeight )
		{
			this->m_btnSize.cy = nHeight;
		}
	}

	//
	// Read and prepare BORDER values!
	//
	plexdata::string::CString sBorder = this->ReadValue
		( CConfiguration::APPLICATION
		, CConfiguration::BORDER
		, CConfiguration::EMPTY
		, szIniFile
		);
	
	if ( !sBorder.IsEmpty() )
	{
		if ( sBorder.Equals( _T("TOOL") ) )
		{
			this->m_nBorder = CConfiguration::BORDER_STYLE_TOOL;
		}
		else if ( sBorder.Equals( _T("THICK") ) )
		{
			this->m_nBorder = CConfiguration::BORDER_STYLE_THICK;
		}
		else if ( sBorder.Equals( _T("THIN") ) )
		{
			this->m_nBorder = CConfiguration::BORDER_STYLE_THIN;
		}
		else if ( sBorder.Equals( _T("NONE") ) )
		{
			this->m_nBorder = CConfiguration::BORDER_STYLE_NONE;
		}
		else
		{
			// Default already set in function SetupDefaults().
		}
	}

	//
	// Read and prepare all configured BUTTON values!
	//
	long nCount = this->GetNumberOfButtonSections( szIniFile );
	
	if ( nCount != 0 )
	{
		this->m_pButtons = new CButtonData[ nCount ];

		if ( this->m_pButtons != NULL )
		{
			this->m_nButtons = nCount;  // Save number of button entries.

			for ( int index=0; index < nCount; index++ )
			{
				plexdata::string::CString sSection;
	
				sSection.Format( CConfiguration::BUTTON, index );

				// Save current button section name.
				this->m_pButtons[index].Section( sSection );

				// Read and set button label.
				this->m_pButtons[index].Label( 
					this->ReadValue
						( sSection.String()
						, CConfiguration::LABEL
						, CConfiguration::EMPTY
						, szIniFile
						)
					);


				// Read and set button tooltip.
				this->m_pButtons[index].Tooltip( 
					this->ReadValue
						( sSection.String()
						, CConfiguration::TOOLTIP
						, CConfiguration::EMPTY
						, szIniFile
						)
					);

				// Read and set button image name.
				this->m_pButtons[index].Image( 
					this->ReadValue
						( sSection.String()
						, CConfiguration::IMAGE
						, CConfiguration::EMPTY
						, szIniFile
						)
					);

				// Read and set button execute string.
				this->m_pButtons[index].Execute( 
					this->ReadValue
						( sSection.String()
						, CConfiguration::EXECUTE
						, CConfiguration::EMPTY
						, szIniFile
						)
					);

				// Read and set button execute parameter string.
				this->m_pButtons[index].Parameter( 
					this->ReadValue
						( sSection.String()
						, CConfiguration::PARAMETER
						, CConfiguration::EMPTY
						, szIniFile
						)
					);
			
				// Read and set button "afterwards" execution string.
				this->m_pButtons[index].Afterwards( 
					this->ReadValue
						( sSection.String()
						, CConfiguration::AFTERWARDS
						, CConfiguration::EMPTY
						, szIniFile
						)
					);

				// Read and set button "afterwards" execution string.
				plexdata::string::CString sAlignment = this->ReadValue
					( sSection.String()
					, CConfiguration::ALIGNMENT
					, CConfiguration::EMPTY
					, szIniFile
					);

				if ( !sAlignment.IsEmpty() )
				{
					if ( sAlignment.Equals( _T("LEFT") ) )
					{
						this->m_pButtons[index].Alignment( DT_LEFT );
					}
					else if ( sAlignment.Equals( _T("RIGHT") ) )
					{
						this->m_pButtons[index].Alignment( DT_RIGHT );
					}
					else
					{
						// Nothing to do because default alignment (DT_CENTER) is used.
					}
				}
			}
		}
	}
}

//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
plexdata::string::CString CConfiguration::ReadValue
	( LPCTSTR szSection
	, LPCTSTR szValue
	, LPCTSTR szDefault
	, LPCTSTR szIniFile
	)
{
	plexdata::string::CString result;

	long   nBuffer = 64;
	LPTSTR pBuffer = result.CreateBuffer( nBuffer );

	while ( true )
	{
		long nRead = ::GetPrivateProfileString
			( szSection
			, szValue
			, szDefault
			, pBuffer
			, nBuffer
			, szIniFile
		);

		if ( nRead == nBuffer || nRead + 1 == nBuffer )
		{
			nBuffer *= 2;
			pBuffer = result.CreateBuffer( nBuffer );
		}
		else
		{
			result.CopyBuffer();
			result.ReleaseBuffer();
			break;
		}
	}

	return result;
}

//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
long CConfiguration::GetNumberOfButtonSections
	( LPCTSTR szIniFile
	)
{
	plexdata::string::CString sSection = CConfiguration::BUTTON;
	plexdata::string::CString sBuffer;
	
	long   nBuffer = 128;
	LPTSTR pBuffer = sBuffer.CreateBuffer( nBuffer );
	long   nRead   = 0;
	long   nCutoff = 0;
	long   nNext   = 0;
	long   nCount  = 0; // Result!

	// Prepare compare string.
	nCutoff  = sSection.FindFirst( _T('%') );
	sSection = sSection.Substring( 0, nCutoff );

	// Get all section names.
	while ( true )
	{
		nRead = ::GetPrivateProfileSectionNames( pBuffer, nBuffer, szIniFile );

		if ( nRead == nBuffer || nRead + 2 == nBuffer )
		{
			nBuffer *= 2;
			pBuffer = sBuffer.CreateBuffer( nBuffer );
		}
		else
		{
			break; // End WHILE loop
		}
	}

	// Count fitting sections.
	for ( int index=0; index < nRead; index++ )
	{
		if ( pBuffer[index] == _T('\0') )
		{
			plexdata::string::CString sTemp( &pBuffer[nNext] );

			nNext = index + 1;

			sTemp = sTemp.Substring( 0, nCutoff );

			if ( sSection.Equals( sTemp ) )
			{
				nCount++; // Count this section.
			}
		}
	}

	return nCount;
}

//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
void CConfiguration::SetupDefaults()
{
	this->m_sTitle   = CConfiguration::EMPTY;
	this->m_sWelcome = CConfiguration::EMPTY;
	
	::DeleteObject( this->m_hBackground );
	this->m_hBackground = NULL;

	::DestroyIcon( this->m_hIcon );
	this->m_hIcon = NULL;

	this->m_nVertical   = CConfiguration::ALIGNMENT_CENTER;
	this->m_nHorizontal = CConfiguration::ALIGNMENT_CENTER;

	this->m_nBorder = CConfiguration::BORDER_STYLE_DEFAULT;

	delete[] this->m_pButtons;
	this->m_pButtons = NULL;
	this->m_nButtons = 0;
}


//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
plexdata::string::CString CConfiguration::GetRandomFile
	( plexdata::string::CString sPath 
	)
{
	plexdata::string::CString result;

	if ( !sPath.IsEmpty() )
	{
		// Ensure path backslash!
		if ( !sPath.EndsWith( _T('\\') ) )
		{
			sPath += _T("\\");
		}

		// Prepare file types to be searched.
		plexdata::string::CString search;
		search.Format( _T("%s%s"), sPath.String(), _T("*.*") );

		WIN32_FIND_DATA sData = {0};
		HANDLE          hFind = ::FindFirstFile( search.String(), &sData );
		int             count = 0;

		// Count number of available files.
		if ( INVALID_HANDLE_VALUE != hFind ) 
		{
			while ( 0 != ::FindNextFile( hFind, &sData ) ) 
			{
				// Avoid directories from being counted.
				if ( !( sData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY ) )
				{
					count++; // Increment file counter.
				}
			}
			::FindClose( hFind );

			// Find a random number within range of available files.
			int random = this->GetRandom( 0, count );

			// Restart with file search.
			hFind = ::FindFirstFile( search.String(), &sData );
			if ( INVALID_HANDLE_VALUE != hFind ) 
			{
				int index = 0;
				while ( 0 != ::FindNextFile( hFind, &sData ) ) 
				{
					// Avoid directories from being counted.
					if ( !( sData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY ) )
					{
						if ( index == random )
						{
							result.Format( _T("%s%s"), sPath.String(), sData.cFileName );
							break;
						}
						index++; // Increment file counter.
					}
				}
				::FindClose( hFind );
			}
		}
	}
	return result;
}

//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////
int CConfiguration::GetRandom
	( int min
	, int max 
	)
{
	// Generates a random number in the half-closed interval 
	// [min, max). In other words, minimum <= random < maximum.

	// Set random seed.
	srand( ::GetTickCount() );

	// Generate "in-range" random number and return.
	return (int)((double)rand() / (RAND_MAX + 1) * (max - min) + min);

}

//////////////////////////////////////////////////////////////////////
// EOF
//////////////////////////////////////////////////////////////////////
