/*
 * Copyright (C) 2010  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#pragma once

#include "string.h"

namespace plexdata
{
	namespace autoplay
	{
		//////////////////////////////////////////////////////////////////////
		// 
		// CButtonData
		//
		//////////////////////////////////////////////////////////////////////
		class CButtonData
		{
		private:

			int m_nAlignment;
			plexdata::string::CString m_sSection;
			plexdata::string::CString m_sLabel;
			plexdata::string::CString m_sTooltip;
			plexdata::string::CString m_sImage;
			plexdata::string::CString m_sExecute;
			plexdata::string::CString m_sParameter;
			plexdata::string::CString m_sAfterwards;

		public:
			
			CButtonData();
			
			virtual ~CButtonData();

		public:

			inline plexdata::string::CString& Section()
			{
				return this->m_sSection;
			}

			inline void Section( plexdata::string::CString& sSection )
			{
				this->m_sSection = sSection;
			}

			inline plexdata::string::CString& Label()
			{
				return this->m_sLabel;
			}

			inline void Label( plexdata::string::CString& sLabel )
			{
				this->m_sLabel = sLabel;
			}

			inline plexdata::string::CString& Tooltip()
			{
				return this->m_sTooltip;
			}

			inline void Tooltip( plexdata::string::CString& sTooltip )
			{
				this->m_sTooltip = sTooltip;
			}

			inline plexdata::string::CString& Image()
			{
				return this->m_sImage;
			}

			inline void Image( plexdata::string::CString& sImage )
			{
				this->m_sImage = sImage;
			}

			inline plexdata::string::CString& Execute()
			{
				return this->m_sExecute;
			}

			inline void Execute( plexdata::string::CString& sExecute )
			{
				this->m_sExecute = sExecute;
			}

			inline plexdata::string::CString& Parameter()
			{
				return this->m_sParameter;
			}

			inline void Parameter( plexdata::string::CString& sParameter )
			{
				this->m_sParameter = sParameter;
			}

			inline plexdata::string::CString& Afterwards()
			{
				return this->m_sAfterwards;
			}

			inline void Afterwards( plexdata::string::CString& sAfterwards )
			{
				this->m_sAfterwards = sAfterwards;
			}

			inline int Alignment()
			{
				return this->m_nAlignment;
			}

			inline void Alignment( int nAlignment )
			{
				this->m_nAlignment = nAlignment;
			}

			inline bool IsValid()
			{
				return ( this->m_sLabel != _T("") && this->m_sExecute != _T("") );
			}

			inline bool IsBrowseAction()
			{
				return this->m_sExecute.Equals( _T("BROWSE") );
			}

			inline bool IsQuitAction()
			{
				return this->m_sExecute.Equals( _T("QUIT") ) || this->m_sExecute.Equals( _T("EXIT") );
			}

			inline bool HasAfterwards()
			{
				return !this->m_sAfterwards.IsEmpty();
			}

		public:

			HBITMAP LoadBitmap();

		};

		//////////////////////////////////////////////////////////////////////
		// 
		// CConfiguration
		//
		//////////////////////////////////////////////////////////////////////
		class CConfiguration
		{
		private:

			static const LPCTSTR EMPTY;
			static const LPCTSTR APPLICATION;
			static const LPCTSTR TITLE;
			static const LPCTSTR BACKGROUND;
			static const LPCTSTR WELCOME;
			static const LPCTSTR ICON;
			static const LPCTSTR ALIGNMENT;
			static const LPCTSTR FONTNAME;
			static const LPCTSTR FONTSIZE;
			static const LPCTSTR FONTCOLOR;
			static const LPCTSTR FONTBOLD;
			static const LPCTSTR BUTTONSIZE;
			static const LPCTSTR BORDER;
			static const LPCTSTR BUTTON;
			static const LPCTSTR LABEL;
			static const LPCTSTR TOOLTIP;
			static const LPCTSTR IMAGE;
			static const LPCTSTR EXECUTE;
			static const LPCTSTR PARAMETER;
			static const LPCTSTR AFTERWARDS;

		public:

			static const int ALIGNMENT_TOP;
			static const int ALIGNMENT_BOTTOM;
			static const int ALIGNMENT_LEFT;
			static const int ALIGNMENT_RIGTH;
			static const int ALIGNMENT_CENTER;

			static const int BORDER_STYLE_DEFAULT;
			static const int BORDER_STYLE_TOOL;
			static const int BORDER_STYLE_THICK;
			static const int BORDER_STYLE_THIN;
			static const int BORDER_STYLE_NONE;

		private:

			plexdata::string::CString m_sTitle;
			plexdata::string::CString m_sWelcome;
			HBITMAP                   m_hBackground;
			HICON                     m_hIcon;
			int                       m_nVertical;
			int                       m_nHorizontal;
			plexdata::string::CString m_sFontName;
			int                       m_nFontSize;
			COLORREF                  m_nFontColor;
			bool                      m_bFontBold;
			SIZE                      m_btnSize;
			int                       m_nBorder;
			CButtonData*              m_pButtons;
			long                      m_nButtons;

		public:

			CConfiguration();

			virtual ~CConfiguration();

		public:

			void Load( LPCTSTR szIniFile, int nIconID );

		public:

			inline LPCTSTR Title()
			{
				return this->m_sTitle.String();
			}

			inline void Title( LPCTSTR szTitle )
			{
				this->m_sTitle = szTitle;
			}

			inline LPCTSTR Welcome()
			{
				return this->m_sWelcome.String();
			}

			inline HBITMAP Background()
			{
				return this->m_hBackground;
			}

			inline HICON Icon()
			{
				return this->m_hIcon;
			}

			inline int VerticalAlignment()
			{
				return this->m_nVertical;
			}

			inline int HorizontalAlignment()
			{
				return this->m_nHorizontal;
			}

			inline LPCTSTR FontName()
			{
				return this->m_sFontName.String();
			}

			inline int FontSize()
			{
				return this->m_nFontSize;
			}

			inline COLORREF FontColor()
			{
				return this->m_nFontColor;
			}

			inline bool FontBold()
			{
				return this->m_bFontBold;
			}

			inline SIZE ButtonSize()
			{
				return this->m_btnSize;
			}

			inline long GetButtonCount()
			{
				return this->m_nButtons;
			}

			inline int GetBorderStyle()
			{
				return this->m_nBorder;
			}

			inline CButtonData* GetButtonEntry( int index )
			{
				if ( 0 <= index && index < this->m_nButtons )
				{
					return &this->m_pButtons[index];
				}
				else
				{
					_ASSERT( false );
					return NULL;
				}
			}

			inline CButtonData* FindAfterwardsButton( const plexdata::string::CString& sSection )
			{
				for ( int index=0; index < this->m_nButtons; index++ )
				{
					if ( this->m_pButtons[index].Section().Equals( const_cast<plexdata::string::CString&>( sSection ) ) )
					{
					return &this->m_pButtons[index];
					}
				}
				_ASSERT( false );
				return NULL;
			}

		private:

			plexdata::string::CString ReadValue
				( LPCTSTR szSection
				, LPCTSTR szValue
				, LPCTSTR szDefault
				, LPCTSTR szIniFile
				);

			long GetNumberOfButtonSections
				( LPCTSTR szIniFile
				);

			void SetupDefaults();

			plexdata::string::CString GetRandomFile
				( plexdata::string::CString sPath 
				);

			int GetRandom
				( int min
				, int max 
				);
		};
	}
}

//////////////////////////////////////////////////////////////////////
// EOF
//////////////////////////////////////////////////////////////////////
