/*
 * Copyright (C) 2010  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#pragma once

namespace plexdata
{
	namespace autoplay
	{
		class CCursor
		{
		public:

			CCursor( TCHAR* szCursorName )
				: m_hInstance(NULL)
				, m_hNewCursor(NULL)
				, m_hOldCursor(NULL)
			{
				this->LoadCursor( szCursorName );
			}

			CCursor( long nCursorID )
				: m_hInstance(NULL)
				, m_hNewCursor(NULL)
				, m_hOldCursor(NULL)
			{
				this->LoadCursor( MAKEINTRESOURCE( nCursorID ) );
			}

			~CCursor()
			{
				this->UnloadCursor();
			}

		public:

			inline void UseCursor()
			{
				::SetCursor( m_hNewCursor );
			}

		protected:

			HINSTANCE m_hInstance;
			HCURSOR   m_hNewCursor;
			HCURSOR   m_hOldCursor;

		protected:

			inline void LoadCursor( TCHAR* szCursorName )
			{
				this->m_hNewCursor = ::LoadCursor( this->m_hInstance, szCursorName );
				this->m_hOldCursor = ::SetCursor( this->m_hNewCursor );
			}

		private:

			inline void UnloadCursor()
			{
				::SetCursor( this->m_hOldCursor );
				::DestroyCursor( this->m_hNewCursor );
				m_hOldCursor = NULL;
				m_hNewCursor = NULL;
			}
		};

		class CWaitCursor : public CCursor
		{
		public:
			CWaitCursor()
				: CCursor( IDC_WAIT )
			{
			}

			virtual ~CWaitCursor()
			{
			};
		};

		class CWaitCursorEx : public CCursor
		{
		public:
			CWaitCursorEx()
				: CCursor( IDC_APPSTARTING )
			{
				// Load IDC_WAIT cursor if IDC_APPSTARTING cursor is not available!
				if ( this->m_hNewCursor == NULL )
				{
					this->LoadCursor( IDC_WAIT );
				}
			}

			virtual ~CWaitCursorEx()
			{
			};
		};

	}
}