/*
 * Copyright (C) 2010  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#pragma once

#include <tchar.h>

namespace plexdata
{
	namespace string
	{
		class CString
		{
			long   m_nString;
			TCHAR* m_szString;
			TCHAR* m_szBuffer;

		public:

			CString();

			CString( const TCHAR* szString );

			CString( CString& rString );

			CString( CString* pString );

			virtual ~CString();

		public:

			inline const TCHAR* String()
			{
				return this->m_szString;
			}

			inline long Length()
			{
				return this->m_nString;
			}

			inline bool IsEmpty()
			{
				return ( 0 == this->Length() );
			}

			inline bool EndsWith( TCHAR cEnd )
			{
				if ( !this->IsEmpty() )
				{
					return ( this->m_szString[this->m_nString-1] == cEnd );
				}
				else 
				{
					return false;
				}
			}

		public:

			inline CString& operator += ( const TCHAR* szString )
			{
				this->Append( szString );
				return *this;
			}

			inline CString& operator += ( CString& rString )
			{
				this->Append( rString );
				return *this;
			}

			inline CString& operator += ( CString* pString )
			{
				this->Append( pString );
				return *this;
			}

			inline CString& operator = ( const TCHAR* szString )
			{
				this->Copy( szString );
				return *this;
			}

			inline CString& operator = ( CString& rString )
			{
				this->Copy( rString );
				return *this;
			}

			inline CString& operator = ( CString* pString )
			{
				this->Copy( pString );
				return *this;
			}

			inline bool operator == ( const TCHAR* szString )
			{
				return this->Equals( szString, false );
			}

			inline bool operator == ( CString& rString )
			{
				return this->Equals( rString, false );
			}

			inline bool operator == ( CString* pString )
			{
				return this->Equals( pString, false );
			}

			inline bool operator != ( const TCHAR* szString )
			{
				return !this->Equals( szString, false );
			}

			inline bool operator != ( CString& rString )
			{
				return !this->Equals( rString, false );
			}

			inline bool operator != ( CString* pString )
			{
				return !this->Equals( pString, false );
			}

		public:

			bool Append( const TCHAR* szString );

			bool Append( CString& rString );

			bool Append( CString* pString );

			bool Copy( const TCHAR* szString );

			bool Copy( CString& rString );

			bool Copy( CString* pString );

			bool Equals( const TCHAR* szString, bool bIgnore=true );

			bool Equals( CString& rString, bool bIgnore=true );

			bool Equals( CString* pString, bool bIgnore=true );

			bool Format( const TCHAR* szFormat, ... );

			CString Substring( long nFirst, long nLast=-1L );

			CString Trim();

			CString LeftTrim();

			CString RightTrim();

			long FindFirst( const TCHAR cFind );

			long FindNext( const TCHAR cFind, long nStart );

			long FindLast( const TCHAR cFind );

			TCHAR GetAt( long index );

			TCHAR* CreateBuffer( long nCount );

			bool CopyBuffer();

			void ReleaseBuffer();

		private:

			bool Create( long nCount );

			bool Create( const TCHAR* szString );

			void Destroy();
		};
	}
}