/*
 * Copyright (C) 2010  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "stdafx.h"
#include "version.h"
#include <windows.h>
#include <crtdbg.h>

#pragma comment( lib, "version.lib")

using namespace plexdata::version;

CVersionInfo::CVersionInfo()
	: m_szFileVersionRoot(NULL)
	, m_szProductVersionRoot(NULL)
	, m_szComments(NULL) 
	, m_szInternalName(NULL) 
	, m_szProductName(NULL) 
	, m_szCompanyName(NULL) 
	, m_szLegalCopyright(NULL) 
	, m_szProductVersion(NULL) 
	, m_szFileDescription(NULL) 
	, m_szLegalTrademarks(NULL) 
	, m_szPrivateBuild(NULL) 
	, m_szFileVersion(NULL) 
	, m_szOriginalFilename(NULL) 
	, m_szSpecialBuild(NULL) 
{
}

CVersionInfo::~CVersionInfo()
{
	delete this->m_szFileVersionRoot;
	delete this->m_szProductVersionRoot;
	delete this->m_szComments; 
	delete this->m_szInternalName; 
	delete this->m_szProductName; 
	delete this->m_szCompanyName; 
	delete this->m_szLegalCopyright; 
	delete this->m_szProductVersion; 
	delete this->m_szFileDescription; 
	delete this->m_szLegalTrademarks; 
	delete this->m_szPrivateBuild; 
	delete this->m_szFileVersion; 
	delete this->m_szOriginalFilename; 
	delete this->m_szSpecialBuild;
}

void CVersionInfo::Initialize( const TCHAR* szFilename )
{
	if ( szFilename != NULL && szFilename[0] != _T('\0') )
	{
		// Allocate buffer to hold file version information.
		unsigned char* pBuffer = NULL;
		unsigned long  nBuffer = ::GetFileVersionInfoSize( szFilename, NULL );

		if ( nBuffer != 0 )
		{
			pBuffer = new unsigned char[nBuffer];

			if ( pBuffer != NULL )
			{
				if ( ::GetFileVersionInfo( szFilename, 0, nBuffer, pBuffer ) )
				{
					this->GetRootInformation( pBuffer );

					TCHAR szLangID[9] = _T("");
					TCHAR nLangID     = sizeof(szLangID) / sizeof(TCHAR);

					if ( this->GetLanguageID( pBuffer, szLangID, nLangID ) )
					{
						this->GetStringValue( pBuffer, szLangID, _T("Comments"),         &this->m_szComments         );
						this->GetStringValue( pBuffer, szLangID, _T("InternalName"),     &this->m_szInternalName     );
						this->GetStringValue( pBuffer, szLangID, _T("ProductName"),      &this->m_szProductName      );
						this->GetStringValue( pBuffer, szLangID, _T("CompanyName"),      &this->m_szCompanyName      );
						this->GetStringValue( pBuffer, szLangID, _T("LegalCopyright"),   &this->m_szLegalCopyright   );
						this->GetStringValue( pBuffer, szLangID, _T("ProductVersion"),   &this->m_szProductVersion   );
						this->GetStringValue( pBuffer, szLangID, _T("FileDescription"),  &this->m_szFileDescription  );
						this->GetStringValue( pBuffer, szLangID, _T("LegalTrademarks"),  &this->m_szLegalTrademarks  );
						this->GetStringValue( pBuffer, szLangID, _T("PrivateBuild"),     &this->m_szPrivateBuild     );
						this->GetStringValue( pBuffer, szLangID, _T("FileVersion"),      &this->m_szFileVersion      );
						this->GetStringValue( pBuffer, szLangID, _T("OriginalFilename"), &this->m_szOriginalFilename );
						this->GetStringValue( pBuffer, szLangID, _T("SpecialBuild"),     &this->m_szSpecialBuild     );
					}
				}
				delete pBuffer;
			}
		}
	}
}

void CVersionInfo::GetRootInformation( void* pBuffer )
{
	void*        pData = NULL;
	unsigned int nData = 0;

	_ASSERT( pBuffer != NULL );

	if ( ::VerQueryValue( pBuffer, _T("\\"), &pData, &nData ) )
	{
		if ( nData == sizeof(VS_FIXEDFILEINFO) )
		{
			VS_FIXEDFILEINFO* pInfo = reinterpret_cast<VS_FIXEDFILEINFO*>( pData );

			_ASSERT( pInfo != NULL );

			if ( pInfo != NULL )
			{
				int nLength = 25; // 5 chars per part + 3 dots + termination + one to satisfy _stprintf_s()

				delete this->m_szFileVersionRoot;
				this->m_szFileVersionRoot = new TCHAR[nLength];
				if ( this->m_szFileVersionRoot != NULL )
				{
					_stprintf_s
						( this->m_szFileVersionRoot
						, nLength
						, _T("%d.%d.%d.%d")
						, HIWORD(pInfo->dwFileVersionMS)
						, LOWORD(pInfo->dwFileVersionMS)
						, HIWORD(pInfo->dwFileVersionLS)
						, LOWORD(pInfo->dwFileVersionLS)
						);
				}

				delete this->m_szProductVersionRoot;
				this->m_szProductVersionRoot = new TCHAR[nLength];
				if ( this->m_szProductVersionRoot != NULL )
				{
					_stprintf_s
						( this->m_szProductVersionRoot
						, nLength
						, _T("%d.%d.%d.%d")
						, HIWORD(pInfo->dwProductVersionMS)
						, LOWORD(pInfo->dwProductVersionMS)
						, HIWORD(pInfo->dwProductVersionLS)
						, LOWORD(pInfo->dwProductVersionLS)
						);
				}
			}
		}
	}
}


bool CVersionInfo::GetLanguageID( void* pBuffer, TCHAR* szResult, int nResult )
{
	bool success = false;

	void*        pData = NULL;
	unsigned int nData = 0;

	_ASSERT( pBuffer  != NULL );
	_ASSERT( szResult != NULL );
	_ASSERT( nResult  >  8    );

	if ( ::VerQueryValue( pBuffer, _T("\\VarFileInfo\\Translation"), &pData, &nData ) )
	{
		unsigned short* pLangID = reinterpret_cast<unsigned short*>( pData );

		_ASSERT( pLangID != NULL );

		success = ( -1 != _stprintf_s
				( szResult
				, nResult
				, _T("%.4X%.4X")
				, pLangID[0]
				, pLangID[1]
				)
			);
	}

	return success;
}


void CVersionInfo::GetStringValue( void* pBuffer, TCHAR* szLangID, TCHAR* szValue, TCHAR** ppResult )
{
	TCHAR        szBlock[50] = _T("");
	void*        pData       = NULL;
	unsigned int nData       = 0;

	_ASSERT( pBuffer     != NULL     );
	_ASSERT( szLangID    != NULL     );
	_ASSERT( szLangID[0] != _T('\0') );
	_ASSERT( szValue     != NULL     );
	_ASSERT( szValue[0]  != _T('\0') );
	_ASSERT( ppResult    != NULL     );

	// Delete original buffer, if is available.
	delete *ppResult;
	*ppResult = NULL;

	_stprintf_s
		( szBlock
		, sizeof(szBlock)/sizeof(TCHAR)
		, _T("\\StringFileInfo\\%s\\%s")
		, szLangID
		, szValue
		);

	// Query the requested value.
	if ( ::VerQueryValue( pBuffer, szBlock, &pData, &nData ) )
	{
		*ppResult = new TCHAR[nData];

		if ( *ppResult != NULL )
		{
			_tcscpy_s( *ppResult, nData, (LPCTSTR)pData );
		}
	}
}
